/**
 * Created by jmoutard on 07/06/16.
 */

/**
 * @description The node properties and attributes.
 * @type {object}
 */
var node_m = {
    _properties_ : {
        me               : "me",

        succeedingEdges  : "succeedingEdges",
        precedingEdges   : "precedingEdges",
        parent           : "parent",
        nodes            : "nodes",
        cytoscapeNodes   : "cytoscapeNodes",

        pre_selected     : "pre_selected",
        selected         : "selected",

        impacted         : "impacted",
        causeOf          : "causeOf",
        filter           : "filter",

        width            : "width",
        height           : "height"
    },
    _attributes_: {
        data    : {
            id               : "id",
            class            : "class",
            code             : "code",
            description      : "description",
            label            : "label",
            redundant        : "redundant",
            hiddenProperties : "hiddenProperties",
            impactNode       : "impactNode",
            parent           : "parent",
            groupbycause     : "groupbycause",
            groupbyimpact    : "groupbyimpact",
            _type            : "_type"
        },
        group       : "group",
        selected    : "selected",
        position    : "position",
        classes     : "classes"
    }
};
/**
 * @description The node properties.
 * @type {node_m._properties_|{me, nodes, succeedingEdges, precedingEdges, parent, filter, impacted, causeOf, pre_selected, selected, width, height, personalPosition, lvl}}
 */
var node_p = node_m._properties_;
/**
 * @description The node attributes.
 * @type {node_m._attributes_|{data, group}}
 */
var node_a = node_m._attributes_;

/**
 * @description The edge properties and attributes.
 * @type {object}
 */
var edge_m = {
    _properties_ : {
        me          : "me",

        source      : "source",
        target      : "target",

        nodes       : "nodes",

        moving      : "_moving"
    },
    _attributes_ : {
        data : {
            id               : "id",
            source           : "source",
            target           : "target",
            criticality      : "criticality",
            hiddenProperties : "hiddenProperties",
            realSource       : "realSource",
            realTarget       : "realTarget"
        },
        group: "group"
    }
};
/**
 * @description The edge properties.
 * @type {edge_m._properties_|{me, source, target, nodes}}
 */
var edge_p = edge_m._properties_;
/**
 * @description The edge attributes.
 * @type {edge_m._attributes_|{source, target, hiddenProperties}}
 */
var edge_a = edge_m._attributes_;

/**
 * @description Create a Node Element. Provide data access functions.
 * @param {object} data
 * @constructor
 */
var GNode = function (data) {
    /*
     * Here is only for documentation for those who use IDE and for the other .js files.
     */
    /* Element function (Parent object). */
    /**
     * @description Setter/Getter of data. To get a specific data give a 'dataName', to set a specific data give also her 'dataValue'. To get all data, give nothing.
     * @param {string} [dataName] The name of the data to Get/Set.
     * @param {*} [dataValue] The value of the data to Set.
     * @returns {*} The 'dataValue' if using as a setter, the value of the specified data or all data.
     */
    this.data = function (dataName, dataValue) {};
    /**
     * @description Give the 'id' of this Node.
     * @returns {*}
     */
    this.id = function () {};
    /**
     * @description Return this basic Element (only his attributes).
     * @returns {object}
     */
    this.basicElement = function () {};
    /**
     * @description Set a specific attribute.
     * @param {string} attrName The name of the attribute to set.
     * @param {*} attrValue The value to set to the attribute.
     * @returns {*} The attribute value.
     */
    this.setAttr = function (attrName, attrValue) {};
    /**
     * @description Get a the value of a specific attribute.
     * @param {string} attrName The name of the attribute to get.
     * @returns {*}
     */
    this.getAttr = function (attrName) {};
    /**
     * @description Remove a specific attribute.
     * @param {string} attrName The name of the attribute to remove.
     * @returns {*} Return the old value of the deleted attribute or undefined if the attribute does'nt exist.
     */
    this.removeAttr = function (attrName) {};
    /**
     * @description Tell if a specific attribute exist (!== undefined).
     * @param {string} attrName The name of the attribute to test.
     * @returns {boolean}
     */
    this.hasAttr = function (attrName) {};
    /**
     * @description Set a specific property.
     * @param {string} propName The name of the property to set.
     * @param {*} propValue The value to set to the property.
     * @returns {*} The value of the property.
     */
    this.setProp = function (propName, propValue) {};
    /**
     * @description Get the value of a specific property.
     * @param {string} propName The name of the property to get.
     * @returns {*}
     */
    this.getProp = function (propName) {};
    /**
     * @description Remove a specific value.
     * @param {string} propName The name of the property to remove.
     * @returns {*} Return the old value of the deleted property or undefined if the property does'nt exist.
     */
    this.removeProp = function (propName) {};
    /**
     * @description Tell if a specific property exist (!== undefined).
     * @param {string} propName The name of the property to test.
     * @returns {boolean}
     */
    this.hasProp = function (propName) {};
    /**
     * @description Add the specified class to this Node.
     * @param {string} className The name of the class to add.
     * @returns {boolean} Return if the job was successfully done. False means that this Node already have this class.
     */
    this.addClass = function (className) {};
    /**
     * @description Remove the specified class from this Node.
     * @param {string} className The class to remove from this Node.
     * @returns {boolean} Return if the job was successfully done. False means that this Node does'nt have the specified class.
     */
    this.removeClass = function (className) {};
    /**
     * @description Test if this Node has the specified class.
     * @param {string} className The name of the class to test.
     * @returns {boolean}
     */
    this.hasClass = function (className) {};
    /**
     * @description Give all of this Node classes.
     * @returns {string[]}
     */
    this.getClasses = function () {};
    /**
     * @description Remove all classes from this Node.
     * @returns {boolean} Return if the job was successfully done. False means that this Node does'nt have any class.
     */
    this.eraseClasses = function () {};
    /**
     * @description Hey Node, have you 'attrName'? Similar to hasAttr.
     * @param {string} attrName
     * @returns {boolean} Return true only if the attribute is a truthy value.
     */
    this.haveYou = function (attrName) {};
    /**
     * @description Hey Node, are you 'propName'? Similar to hasProp.
     * @param {string} propName
     * @returns {boolean} Return true only if the property is a truthy value.
     */
    this.areYou = function (propName)  {};
    /* GNode functions. */
    /**
     * @description Select me.
     */
    this.select = function () {};
    /**
     * @description Unselect me.
     */
    this.unselect = function () {};
    /**
     * @description Am I selected?
     * @returns {boolean}
     */
    this.selected = function () {};
    /**
     * @description Give the preceding GEdge of this GNode.
     * @returns {GEdge[]}
     */
    this.precedingEdges = function () {};
    /**
     * @description Give the succeeding GEdge of this GNode.
     * @returns {GEdge[]}
     */
    this.succeedingEdges = function () {};
    /**
     * @description Give the preceding GNode of this GNode.
     * @returns {GNode[]}
     */
    this.precedingNodes = function () {};
    /**
     * @description Give the succeeding GNode of this GNode.
     * @returns {GNode[]}
     */
    this.succeedingNodes = function () {};
    /**
     * @description Tell if this GNode has predecessors.
     * @returns {boolean}
     */
    this.hasPredecessors = function () {};
    /**
     * @description Tell if this GNode has successors.
     * @returns {boolean}
     */
    this.hasSuccessors = function () {};
};
/**
 * @description Create a Edge Element. Provide data access functions.
 * @param {object} data
 * @constructor
 */
var GEdge = function (data) {
    /*
     * Here is only for documentation for those who use IDE and for the other .js files.
     */
    /* Element functions (Parent object). */
    /**
     * @description Setter/Getter of data. To get a specific data give a 'dataName', to set a specific data give also her 'dataValue'. To get all data, give nothing.
     * @param {string} [dataName] The name of the data to Get/Set.
     * @param {*} [dataValue] The value of the data to Set.
     * @returns {*} The 'dataValue' if using as a setter, the value of the specified data or all data.
     */
    this.data = function (dataName, dataValue) {};
    /**
     * @description Give the 'id' of this Edge.
     * @returns {*}
     */
    /* this.id = function () {}; */ /* A edge does'nt have an id. */
    /**
     * @description Return the basic Edge (only his attributes).
     * @returns {object}
     */
    this.basicElement = function () {};
    /**
     * @description Set a specific attribute.
     * @param {string} attrName The name of the attribute to set.
     * @param {*} attrValue The value to set to the attribute.
     * @returns {*} The attribute value.
     */
    this.setAttr = function (attrName, attrValue) {};
    /**
     * @description Get a the value of a specific attribute.
     * @param {string} attrName The name of the attribute to get.
     * @returns {*}
     */
    this.getAttr = function (attrName) {};
    /**
     * @description Remove a specific attribute.
     * @param {string} attrName The name of the attribute to remove.
     * @returns {*} Return the old value of the deleted attribute or undefined if the attribute does'nt exist.
     */
    this.removeAttr = function (attrName) {};
    /**
     * @description Tell if a specific attribute exist (!== undefined).
     * @param {string} attrName The name of the attribute to test.
     * @returns {boolean}
     */
    this.hasAttr = function (attrName) {};
    /**
     * @description Set a specific property.
     * @param {string} propName The name of the property to set.
     * @param {*} propValue The value to set to the property.
     * @returns {*} The value of the property.
     */
    this.setProp = function (propName, propValue) {};
    /**
     * @description Get the value of a specific property.
     * @param {string} propName The name of the property to get.
     * @returns {*}
     */
    this.getProp = function (propName) {};
    /**
     * @description Remove a specific value.
     * @param {string} propName The name of the property to remove.
     * @returns {*} Return the old value of the deleted property or undefined if the property does'nt exist.
     */
    this.removeProp = function (propName) {};
    /**
     * @description Tell if a specific property exist (!== undefined).
     * @param {string} propName The name of the property to test.
     * @returns {boolean}
     */
    this.hasProp = function (propName) {};
    /**
     * @description Add the specified class to this Edge.
     * @param {string} className The name of the class to add.
     * @returns {boolean} Return if the job was successfully done. False means that this Edge already have this class.
     */
    this.addClass = function (className) {};
    /**
     * @description Remove the specified class from this Edge.
     * @param {string} className The class to remove from this Edge.
     * @returns {boolean} Return if the job was successfully done. False means that this Edge does'nt have the specified class.
     */
    this.removeClass = function (className) {};
    /**
     * @description Test if this Edge has the specified class.
     * @param {string} className The name of the class to test.
     * @returns {boolean}
     */
    this.hasClass = function (className) {};
    /**
     * @description Give all of this Edge classes.
     * @returns {string[]}
     */
    this.getClasses = function () {};
    /**
     * @description Remove all classes from this Edge.
     * @returns {boolean} Return if the job was successfully done. False means that this Edge does'nt have any class.
     */
    this.eraseClasses = function () {};
    /**
     * @description Hey Edge, have you 'attrName'? Similar to hasAttr.
     * @param {string} attrName
     * @returns {boolean} Return true only if the attribute is a truthy value.
     */
    this.haveYou = function (attrName) {};
    /**
     * @description Hey Edge, are you 'propName'? Similar to hasProp.
     * @param {string} propName
     * @returns {boolean} Return true only if the property is a truthy value.
     */
    this.areYou = function (propName)  {};
    /* GEdge functions. */
    /**
     * @description Give the GNode source of this GEdge.
     * @returns {GNode}
     */
    this.source = function () {};
    /**
     * @description Give the GNode target of this GEdge.
     * @returns {GNode}
     */
    this.target = function () {};
};

var getElements = (function () {
    'use strict';
    /**
     * @description A basic Element.
     * @param {object} data
     * @constructor
     * @private
     */
    function Element (data) {
        /**
         * @description This Element.
         * @type {Element}
         * @private
         */
        var _self = this;
        /**
         * @description The attributes of this Element.
         * @type {object}
         * @private
         */
        var _attributes = data;
        /**
         * @description The properties of this Element.
         * @type {object}
         * @private
         */
        var _properties = {};
        /**
         * @description The css classes of this Element.
         * @type {string[]}
         */
        var _classes = [];

        /**
         * @description Setter/Getter of data. To get a specific data give a 'dataName', to set a specific data give also her 'dataValue'. To get all data, give nothing.
         * @param {string} [dataName] The name of the data to Get/Set.
         * @param {*} [dataValue] The value of the data to Set.
         * @returns {*} The 'dataValue' if using as a setter, the value of the specified data or all data.
         */
        this.data = function (dataName, dataValue) {
            if (dataValue !== undefined) {
                _attributes.data[dataName] = dataValue;
                return dataValue;
            } else if (dataName !== undefined) {
                return _attributes.data[dataName];
            } else {
                return _attributes.data;
            }
        };
        /**
         * @description Give the 'id' of this Element.
         * @returns {*}
         */
        this.id = function () { return _attributes.data.id; };
        /**
         * @description Return the basic Element (only his attributes).
         * @returns {object}
         */
        this.basicElement = function () { return _attributes; };

        /**
         * @description Set a specific attribute.
         * @param {string} attrName The name of the attribute to set.
         * @param {*} attrValue The value to set to the attribute.
         * @returns {*} The attribute value.
         */
        this.setAttr = function (attrName, attrValue) {
            _attributes[attrName] = attrValue;
            return attrValue;
        };
        /**
         * @description Get a the value of a specific attribute.
         * @param {string} attrName The name of the attribute to get.
         * @returns {*}
         */
        this.getAttr = function (attrName) { return _attributes[attrName]; };
        /**
         * @description Remove a specific attribute.
         * @param {string} attrName The name of the attribute to remove.
         * @returns {*} Return the old value of the deleted attribute or undefined if the attribute does'nt exist.
         */
        this.removeAttr = function (attrName) {
            if (_self.hasAttr(attrName)) {
                var oldValue = _attributes[attrName];
                delete _attributes[attrName];
                return oldValue;
            } else {
                return undefined;
            }
        };
        /**
         * @description Tell if a specific attribute exist (!== undefined).
         * @param {string} attrName The name of the attribute to test.
         * @returns {boolean}
         */
        this.hasAttr = function (attrName) { return _attributes[attrName] !== undefined; };

        /**
         * @description Set a specific property.
         * @param {string} propName The name of the property to set.
         * @param {*} propValue The value to set to the property.
         * @returns {*} The value of the property.
         */
        this.setProp = function (propName, propValue) {
            _properties[propName] = propValue;
            return propValue;
        };
        /**
         * @description Get the value of a specific property.
         * @param {string} propName The name of the property to get.
         * @returns {*}
         */
        this.getProp = function (propName) { return _properties[propName]; };
        /**
         * @description Remove a specific value.
         * @param {string} propName The name of the property to remove.
         * @returns {*} Return the old value of the deleted property or undefined if the property does'nt exist.
         */
        this.removeProp = function (propName) {
            if (_self.hasProp(propName)) {
                var oldValue = _properties[propName];
                delete _properties[propName];
                return oldValue;
            } else {
                return undefined;
            }
        };
        /**
         * @description Tell if a specific property exist (!== undefined).
         * @param {string} propName The name of the property to test.
         * @returns {boolean}
         */
        this.hasProp = function (propName) { return _properties[propName] !== undefined; };

        /**
         * @description Add the specified class to this Element.
         * @param {string} className The name of the class to add.
         * @returns {boolean} Return if the job was successfully done. False means that this Element already have this class.
         */
        this.addClass = function (className) {
            if (!_self.hasClass(className)) {
                _classes.push(className);
                return true;
            } else {
                return false;
            }
        };
        /**
         * @description Remove the specified class from this Element.
         * @param {string} className The class to remove from this Element.
         * @returns {boolean} Return if the job was successfully done. False means that this Element does'nt have the specified class.
         */
        this.removeClass = function (className) {
            if (_self.hasClass(className)) {
                _classes.splice(_classes.indexOf(className), 1);
                return true;
            } else {
                return false;
            }
        };
        /**
         * @description Test if this Element has the specified class.
         * @param {string} className The name of the class to test.
         * @returns {boolean}
         */
        this.hasClass = function (className) {
            return _classes.indexOf(className) !== -1;
        };
        /**
         * @description Give all of this Element classes.
         * @returns {string[]}
         */
        this.getClasses = function () { return _classes; };
        /**
         * @description Remove all classes from this Element.
         * @returns {boolean} Return if the job was successfully done. False means that this Element does'nt have any class.
         */
        this.eraseClasses = function () {
            if (_classes.length > 0) {
                _classes = [];
                return true;
            } else {
                return false;
            }
        };

        this.data(node_a.data.hiddenProperties, {me: this, classes: _classes});
    }

    /**
     * @description Hey Element, have you 'attrName'? Similar to hasAttr.
     * @param {string} attrName
     * @returns {boolean} Return true only if the attribute is a truthy value.
     */
    Element.prototype.haveYou = function (attrName) {
        return this.hasAttr(attrName) && (this.getAttr(attrName) ? true : false);
    };
    /**
     * @description Hey Element, are you 'propName'? Similar to hasProp.
     * @param {string} propName
     * @returns {boolean} Return true only if the property is a truthy value.
     */
    Element.prototype.areYou = function (propName)  {
        return this.hasProp(propName) && (this.getProp(propName) ? true: false);
    };

    /**
     * @description A GNode Element.
     * @param {object} data
     * @constructor
     */
    function GNode (data) {
        /**
         * @description This GNode.
         * @type {GNode}
         * @private
         */
        var _self = this;
        Element.call(this, data);

        /**
         * @description Select me.
         */
        this.select = function () {
            _self.setProp(node_p.impacted, _self.setProp(node_p.selected, true));
            _self.data(node_a.data.impactNode, parseInt(myImpactLevel));
        };
        /**
         * @description Unselect me.
         */
        this.unselect = function () {
            _self.removeProp(node_p.impacted);
            _self.removeProp(node_p.selected);
            delete _self.data()[node_a.data.impactNode];
        };
        /**
         * @description Am I selected?
         * @returns {boolean}
         */
        this.selected = function () {
            return _self.areYou(node_p.selected);
        };

        /**
         * @description Hey node, show me what you are!
         * @returns {string}
         */
        this.show = function () {
            var response = "<ul><li><h3>Code</h3><p>" + this.data(node_a.data.code) + "</p></li>";

            if (debug) {
                for (var index in node_p) {
                    if (this.areYou(node_p[index])) {
                        switch (node_p[index]) {
                            case node_p.me:
                            case node_p.cytoscapeNodes:
                                break;
                            case node_p.parent:
                                response += "<li><h5>Has parents</h5>";
                                if (this.getProp(node_p.parent)[utilisation.cause]) {
                                    response += "<p>When grouped by cause: " + this.getProp(node_p.parent)[utilisation.cause] + "</p></li>";
                                }
                                if (this.getProp(node_p.parent)[utilisation.impact]) {
                                    response += "<p>When grouped by impact: " + this.getProp(node_p.parent)[utilisation.impact] + "</p></li>";
                                }
                                break;
                            case node_p.nodes:
                                response += "<li><h5>Is a group node</h5><p>With " + this.getProp(node_p.nodes).length + " nodes in.</p></li>";
                                break;
                            case node_p.precedingEdges:
                                response += "<li><h5>Has preceding edges</h5>";
                                var precedingEdges = this.precedingEdges();
                                response += "<p><table style='border=1px black solid;'>" +
                                    "<tr><th>Edge source</th><th>Edge target</th><th>Criticality</th></tr>";
                                for (var pe = 0, precedingEdge = precedingEdges[pe]; pe < precedingEdges.length; pe++, precedingEdge = precedingEdges[pe]) {
                                    response += "<tr><td>" + precedingEdge.source().id() + "</td><td>" + precedingEdge.target().id() + "</td><td>" + precedingEdge.data(edge_a.data.criticality) + "</td></tr>";
                                }
                                response += "</table></p></li>";
                                break;
                            case node_p.succeedingEdges:
                                response += "<li><h5>Has " + this.succeedingEdges().length + " succeeding edge(s).</h5>";
                                break;
                            case node_p.impacted:
                                response += "<li><h5>Is impacted</h5>" +
                                    "<p>With an impact of: " + (this.data(node_a.data.impactNode) || this.data(node_a.data.impactNode) === 0 ? this.data(node_a.data.impactNode)  + "%" : this.data(node_a.data.impactNode)) + "</p></li>";
                                break;
                            case node_p.causeOf:
                                var cause = this.getProp(node_p.causeOf);
                                response += "<li><h5>Is cause of:</h5>";
                                if (cause.length > 1) {
                                    response += "<p>1 node so it is just a simple cause.</p></li>";
                                } else {
                                    response += "<p>" + cause.length + " nodes so it is an intersection.</p></li>";
                                }
                                break;
                            case node_p.filter:
                                response += "<li><h5>Is Filtered</h5></li>";
                                break;
                            default:
                                response += "<li><h5>" + index.substring(0,1).toUpperCase() + index.substring(1).toLowerCase() + "</h5>" +
                                    "<p>" + this.getProp(node_p[index]) + "</p></li>";
                                break;
                        }
                    }
                }
                var data = this.data();
                for (var index in data) {
                    if (index.search(/^nb_/) !== -1) {
                        response += "<li><h5>" + index.substring(0,1).toUpperCase() + index.substring(1).toLowerCase() + "</h5>" +
                            "<p>" + data[index] + "</p></li>";
                    } else if (index === "redundant") {
                        response += "<li><h5>Redundant</h5><p>" + data[index] + "</p></li>";
                    }
                }
                response += "<li><h5>Classes</h5><p>" + this.getClasses().join(" ") + "</p></li>";
            } else {
                var data = this.data();
                for (var index in data) {
                    switch (index) {
                        case node_a.data.id:
                        case node_a.data._type:
                        case node_a.data.code:
                        case node_a.data.hiddenProperties:
                        case node_a.data.class:
                        case node_a.data.description:
                        case node_a.data.parent:
                        case node_a.data.groupbycause:
                        case node_a.data.groupbyimpact:
                            break;
                        default:
                            response += "<li><h5>" + index.substring(0,1).toUpperCase() + index.substring(1).toLowerCase() + "</h5>" +
                                "<p>" + data[index] + "</p></li>";
                            break;
                    }
                }
            }

            return response + "</ul>";
        };
    }
    GNode.prototype = Object.create(Element.prototype);
    GNode.prototype.constructor = GNode;

    /**
     * @description Give the preceding GEdge of this GNode.
     * @returns {GEdge[]}
     */
    GNode.prototype.precedingEdges = function () { return this.getProp(node_p.precedingEdges); };
    /**
     * @description Give the succeeding GEdge of this GNode.
     * @returns {GEdge[]}
     */
    GNode.prototype.succeedingEdges = function () { return this.getProp(node_p.succeedingEdges); };
    /**
     * @description Give the preceding GNode of this GNode.
     * @returns {GNode[]}
     */
    GNode.prototype.precedingNodes = function () {
        var precedingNodes = [];
        for (var precedingEdges = this.precedingEdges(), se = 0, precedingEdge = precedingEdges[se];
             se < precedingEdges.length;
             se++, precedingEdge = precedingEdges[se])
        {
            precedingNodes.push(precedingEdge.source());
        }
        return precedingNodes;
    };
    /**
     * @description Give the succeeding GNode of this GNode.
     * @returns {GNode[]}
     */
    GNode.prototype.succeedingNodes = function () {
        var succeedingNodes = [];
        for (var succeedingEdges = this.succeedingEdges(), se = 0, succeedingEdge = succeedingEdges[se];
                                    se < succeedingEdges.length;
                            se++, succeedingEdge = succeedingEdges[se])
        {
            succeedingNodes.push(succeedingEdge.target());
        }
        return succeedingNodes;
    };

    /**
     * @description Tell if this GNode has predecessors.
     * @returns {boolean}
     */
    GNode.prototype.hasPredecessors = function () { return this.hasProp(node_p.precedingEdges) && this.getProp(node_p.precedingEdges).length > 0; };
    /**
     * @description Tell if this GNode has successors.
     * @returns {boolean}
     */
    GNode.prototype.hasSuccessors = function () { return this.hasProp(node_p.succeedingEdges) && this.getProp(node_p.succeedingEdges).length > 0; };

    /**
     * @description A GEdge Element.
     * @param {object} data
     * @constructor
     */
    function GEdge (data) {
        Element.call(this, data);

        /**
         * @description Hey edge, show me what you are!
         * @returns {string}
         */
        this.show = function () {
            var response = "<ul>";
            if (debug) {
                for (var index in edge_p) {
                    response += "<li>";
                    switch (edge_p[index]) {
                        case edge_p.me:
                        case edge_p.moving:
                            break;
                        case edge_p.source:
                            response += "<h5>Source</h5><p>" + this.source().id() + "</p>";
                            break;
                        case edge_p.target:
                            response += "<h5>Target</h5><p>" + this.target().id() + "</p>";
                            break;
                        case edge_p.nodes:
                            if (this.areYou(edge_p.nodes)) {
                                response += "<h5>Filter nodes</h5><p>" + this.getProp(edge_p.nodes).length + " node(s).</p>";
                            }
                            break;
                    }
                    response += "</li>";
                }
            } else {
                var data = this.data();
                for (var index in data) {
                    response += "<li>";
                    switch (index) {
                        case edge_a.data.hiddenProperties:
                        case edge_a.data.id:
                        case edge_a.data.realSource:
                        case edge_a.data.realTarget:
                            break;
                        default:
                            response += "<h5>" + index.substring(0,1).toUpperCase() + index.substring(1).toLowerCase() + "</h5>" +
                                "<p>" + this.data(index) + "</p>";
                            break;
                    }
                    response += "</li>";
                }
            }
            return response + "</ul>";
        }
    }
    GEdge.prototype = Object.create(Element.prototype);
    GEdge.prototype.constructor = GEdge;

    /**
     * @description Give the GNode source of this GEdge.
     * @returns {GNode}
     */
    GEdge.prototype.source = function () { return this.getProp(edge_p.source); };
    /**
     * @description Give the GNode target of this GEdge.
     * @returns {GNode}
     */
    GEdge.prototype.target = function () { return this.getProp(edge_p.target); };

    return {GNode: GNode, GEdge: GEdge};
}());

GNode = getElements.GNode;
GEdge = getElements.GEdge;
getElements = undefined;

/**
 * @description Provide Element manager.
 */
var ele = (function () {
    'use strict';
    /**
     * @description Create a manager for this element and using this json.
     * @param {{data:{id:string, source:string, target:string}, group:string}} element
     * @param {{nodes:object, edges:object[], groups:object[]}|object[]} json
     * @returns {ElementManager}
     * @constructor
     */
    function Element (element, json) { return new ElementManager(element, json); }

    /**
     * @description Manage an element using a json.
     * @param {{data:{id:string, source:string, target:string}, group:string}} element
     * @param {{nodes:object, edges:object[], groups:object[]}|object[]} json
     * @constructor
     */
    function ElementManager (element, json) {
        /**
         * @description This ElementManager.
         * @type {ElementManager}
         * @private
         */
        var _self = this;
        /**
         * @description The element.
         * @type {{data:{id:string, source:string, target:string}, group:string}}
         * @private
         */
        var _element = element;
        /**
         * @description The json to loop in.
         * @type {{nodes:object, edges:object[], groups:object[]}|object[]}
         * @private
         */
        var _json = json;
        /**
         * @description Tell if this '_element' is a node.
         * @type boolean
         */
        var _is_node = Element.is(_element, elementGroup.node);

        /**
         * @description Tell if this '_element' is really this type.
         * @param {string} type "nodes" or "edges".
         * @returns {boolean}
         */
        this.is = function (type) {
            return _element.group === type;
        };

        /**
         * @description Get all the preceding edges of this '_element' from that json.
         * @returns {object[]}
         */
        this.precedingEdges = function () {
            var response = [],
                iterator = new IteratorGraph((_json.edges ? _json.edges : _json));

            iterator.forEach(function (element) {
                if (Element.is(element, elementGroup.edge) && element.data.target === (_is_node ? _element.data.id : _element.data.source)) {
                    response.push(element);
                }
            });

            return response;
        };
        /**
         * @description Get all the succeeding edges of this '_element' from that json.
         * @returns {object[]}
         */
        this.succeedingEdges = function () {
            var response = [],
                iterator = new IteratorGraph((_json.edges ? _json.edges : _json));

            iterator.forEach(function (element) {
                if (Element.is(element, elementGroup.edge) && element.data.source === (_is_node ? _element.data.id : _element.data.target)) {
                    response.push(element);
                }
            });

            return response;
        };
        /**
         * @description Get all the preceding nodes of this '_element' from that json.
         * @returns {object[]}
         */
        this.precedingNodes = function () {
            var response = [], iterator;

            if (_is_node) {
                iterator = new IteratorGraph(_self.precedingEdges());
            } else {
                iterator = new IteratorGraph((_json.nodes ? _json.nodes : _json));
            }

            iterator.forEach(function (element) {
                if (_is_node) {
                    Array.prototype.push.apply(response, Element.precedingNodes(element, json));
                } else if (Element.is(element, elementGroup.node) && element.data.id === _element.data.source) {
                    response.push(element);
                }
            });

            return response;
        };
        /**
         * @description Get all the succeeding nodes of this '_element' from that json.
         * @returns {object[]}
         */
        this.succeedingNodes = function () {
            var response = [], iterator;

            if (_is_node) {
                iterator = new IteratorGraph(_self.succeedingEdges());
            } else {
                iterator = new IteratorGraph((_json.nodes ? _json.nodes : _json));
            }

            iterator.forEach(function (element) {
                if (_is_node) {
                    Array.prototype.push.apply(response, Element.succeedingNodes(element, json));
                } else if (Element.is(element, elementGroup.node) && element.data.id === _element.data.target) {
                    response.push(element);
                }
            });

            return response;
        };
    }

    /**
     * @description Tell if this element's group is the same as this type.
     * @param {{group:string}} element
     * @param {string} type
     * @returns {boolean}
     */
    Element.is = function (element, type) {
        return element.group === type;
    };

    /**
     * @description Give all of this element's preceding edge from that json.
     * @param {{data:{id:string, source:string}, group:string}} element
     * @param {{nodes:object, edges:object[], groups:object[]}|object[]} json
     * @returns {object[]}
     */
    Element.precedingEdges = function(element, json) {
        var _is_node = Element.is(element, elementGroup.node),
            iterator = new IteratorGraph((json.edges ? json.edges : json)),
            response = [];

        iterator.forEach(function (element_) {
            if (Element.is(element_, elementGroup.edge) && element_.data.target === (_is_node ? element.data.id : element.data.source)) {
                response.push(element_);
            }
        });

        return response;
    };
    /**
     * @description Give all of this element's succeeding edge from that json.
     * @param {{data:{id:string, source:string}, group:string}} element
     * @param {{nodes:object, edges:object[], groups:object[]}|object[]} json
     * @returns {object[]}
     */
    Element.succeedingEdges = function(element, json) {
        var _is_node = Element.is(element, elementGroup.node),
            iterator = new IteratorGraph((json.edges ? json.edges : json)),
            response = [];

        iterator.forEach(function (element_) {
            if (Element.is(element_, elementGroup.edge) && element_.data.source === (_is_node ? element.data.id : element.data.target)) {
                response.push(element_);
            }
        });

        return response;
    };
    /**
     * @description Give all of this element's preceding node from that json.
     * @param {{data:{id:string, source:string}, group:string}} element
     * @param {{nodes:object, edges:object[], groups:object[]}|object[]} json
     * @returns {object[]}
     */
    Element.precedingNodes = function(element, json) {
        var _is_node = Element.is(element, elementGroup.node),
            iterator,
            response = [];

        if (_is_node) {
            iterator = new IteratorGraph(Element.precedingEdges(element, json));
        } else {
            iterator = new IteratorGraph((json.edges ? json.edges : json));
        }

        iterator.forEach(function (element_) {
            if (_is_node) {
                Array.prototype.push.apply(response, Element.precedingNodes(element_, json));
            } else if (Element.is(element_, elementGroup.edge) && element_.data.source === element.data.target) {
                response.push(element_);
            }
        });

        return response;
    };
    /**
     * @description Give all of this element's succeeding node from that json.
     * @param {{data:{id:string, source:string}, group:string}} element
     * @param {{nodes:object, edges:object[], groups:object[]}|object[]} json
     * @returns {object[]}
     */
    Element.succeedingNodes = function(element, json) {
        var _is_node = Element.is(element, elementGroup.node),
            iterator,
            response = [];

        if (_is_node) {
            iterator = new IteratorGraph(Element.precedingEdges(element, json));
        } else {
            iterator = new IteratorGraph((json.edges ? json.edges : json));
        }

        iterator.forEach(function (element_) {
            if (_is_node) {
                Array.prototype.push.apply(response, Element.succeedingNodes(element_, json));
            } else if (Element.is(element_, elementGroup.edge) && element_.data.source === element.data.target) {
                response.push(element_);
            }
        });

        return response;
    };
    /**
     * @description Tell if this is really that element.
     * @param {Object} element The element to test.
     * @param {String} id The wanted element id.
     * @returns {boolean}
     */
    Element.isItHim = function (element, id) {
        return element.data.id === id;
    };

    return Element;
}());

var GElement = (function () {
    'use strict';
    /**
     *
     * @param {GNode|GEdge} element
     * @param {GNode[]|GEdge[]} json
     * @constructor
     */
    function GElement (element, json) {
        /**
         * @description The element to refer.
         * @type {GNode|GEdge}
         * @private
         */
        var _element = element;
        /**
         * @description A json to loop in.
         * @type {GNode[]|GEdge[]}
         * @private
         */
        var _json = json;

        /**
         * @description Hey element, is it you?
         * @param {string} id
         * @returns {boolean} Tell if this element id is this 'id'.
         */
        this.isItYou = function (id) {
            return _element.id() === id;
        };
        /**
         * @description Hey element, do you really belongs to this type?
         * @param {string} type
         * @returns {boolean} Tell if this element 'groups' attribute is this 'type'.
         */
        this.typeOf = function (type) {
            /* Type for nodes and edges are in _attributes.groups */
            return _element.getAttr(node_a.group) === type;
        };
        /**
         * @description Only if this element is a node. Give the preceding edges of this element in this json.
         * @returns {Array} Empty if no match.
         */
        this.precedingEdges = function () {
            var response = [];
            if (this.typeOf(elementGroup.node)) {
                new IteratorGraph((_json.edges ? _json.edges : _json)).forEach(
                    /**
                     * @param {GNode|GEdge} element_
                     */
                    function (element_) {
                        if (element_.getAttr(edge_a.group) === elementGroup.edge) {
                            if (element_.data(edge_a.data.target) === _element.id())
                                response.push(element_);
                        }
                    }
                );
            }
            return response;
        };
        /**
         * @description Only if this element is a node. Give the succeeding edges of this element in this json.
         * @returns {Array} Empty if no match.
         */
        this.succeedingEdges = function () {
            var response = [];
            if (this.typeOf(elementGroup.node)) {
                new IteratorGraph((_json.edges ? _json.edges : _json)).forEach(
                    /**
                     * @param {GNode|GEdge} element_
                     */
                    function (element_) {
                        if (element_.getAttr(edge_a.group) === elementGroup.edge) {
                            if (element_.data(edge_a.data.source) === _element.id())
                                response.push(element_);
                        }
                    }
                );
            }
            return response;
        };
    }

    return GElement;
}());